package com.fjhx.mes.service.production.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.dynamic.datasource.annotation.DSTransactional;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.common.utils.Assert;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.mes.entity.production.dto.ProductionSchedulingDto;
import com.fjhx.mes.entity.production.dto.ProductionSchedulingSelectDto;
import com.fjhx.mes.entity.production.po.ProductionOrder;
import com.fjhx.mes.entity.production.po.ProductionOrderDetail;
import com.fjhx.mes.entity.production.po.ProductionScheduling;
import com.fjhx.mes.entity.production.vo.ProductionOrderDetailVo;
import com.fjhx.mes.entity.production.vo.ProductionSchedulingVo;
import com.fjhx.mes.entity.technology.po.Technology;
import com.fjhx.mes.mapper.production.ProduceOrderDetailMapper;
import com.fjhx.mes.mapper.production.ProductionSchedulingMapper;
import com.fjhx.mes.service.production.ProduceOrderDetailService;
import com.fjhx.mes.service.production.ProductionSchedulingService;
import com.fjhx.mes.service.technology.TechnologyService;
import com.fjhx.sale.entity.contract.po.ContractProduct;
import com.fjhx.sale.entity.contract.vo.ContractProductBomVo;
import com.fjhx.sale.service.contract.ContractProductBomService;
import com.fjhx.sale.service.contract.ContractProductService;
import com.fjhx.wms.entity.stock.emums.JournalType;
import com.fjhx.wms.entity.stock.po.StockWait;
import com.fjhx.wms.entity.stock.po.StockWaitDetails;
import com.fjhx.wms.service.stock.StockWaitDetailsService;
import com.fjhx.wms.service.stock.StockWaitService;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.service.ISysUserService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 生产排程 服务实现类
 * </p>
 *
 * @since 2024-02-03
 */
@Service
public class ProductionSchedulingServiceImpl extends ServiceImpl<ProductionSchedulingMapper, ProductionScheduling> implements ProductionSchedulingService {

    @Resource
    private ProductInfoService productInfoService;
    @Resource
    private ProduceOrderDetailService produceOrderDetailService;
    @Resource
    private ProduceOrderDetailMapper produceOrderDetailMapper;
    @Resource
    private ISysUserService sysUserService;
    @Resource
    private StockWaitService stockWaitService;
    @Resource
    private StockWaitDetailsService stockWaitDetailsService;
    @Resource
    private ContractProductBomService contractProductBomService;
    @Resource
    private ContractProductService contractProductService;
    @Resource
    private TechnologyService technologyService;

    @Override
    public Page<ProductionSchedulingVo> getPage(ProductionSchedulingSelectDto dto) {
        IWrapper<ProductionScheduling> wrapper = getWrapper();

        //===============================================================================================
        //业务公司过滤
        wrapper.eq("c.of_company_id", dto.getContractCompanyId());
        //生产公司
        wrapper.eq("po", ProductionOrder::getCompanyId, dto.getCompanyId());
        //生产状态
        String produceStatus = dto.getProduceStatus();
        if (ObjectUtil.isNotEmpty(produceStatus)) {
            wrapper.in("pod", ProductionOrderDetail::getProduceStatus, produceStatus.split(","));
        }
        //是否逾期过滤
        if (ObjectUtil.isNotEmpty(dto.getIsOverdue())) {
            wrapper.eq("IF(IFNULL(po.finish_time,NOW())>po.delivery_period,1,0)", dto.getIsOverdue());
        }
        //交期-时间范围过滤
        wrapper.ge("po", ProductionOrder::getDeliveryPeriod, dto.getStaDeliveryPeriod());
        wrapper.le("po", ProductionOrder::getDeliveryPeriod, dto.getEndDeliveryPeriod());

        //关键字
        if (ObjectUtil.isNotEmpty(dto.getKeyword())) {
            List<SysUser> list = sysUserService.list(IWrapper.<SysUser>getWrapper().like(SysUser::getNickName, dto.getKeyword()));
            List<Long> uIds = list.stream().map(SysUser::getUserId).collect(Collectors.toList());
            wrapper.and(q -> q
                    .like("po.code", dto.getKeyword())
                    .or().in("c.create_user", uIds)
            );
        }
        //======================================================================================================

        //只显示投产的数据
        wrapper.isNotNull("po.produce_time");
        //工序id过滤
        wrapper.eq(ProductionScheduling::getProcessesId, dto.getProcessesId());

        //权限过滤：生产排程-1
        wrapper.in("po", ProductionOrder::getCompanyId, SecurityUtils.getCompanyIds());
        wrapper.eq("po", ProductionOrder::getCompanyId, dto.getCompanyId());

        //排程日期过滤
        wrapper.eq("ps", ProductionScheduling::getSchedulingDate, DateUtil.format(dto.getSchedulingDate(), "yyyy-MM-dd"));

        wrapper.orderByDesc("ps.scheduling_date");

        Page<ProductionSchedulingVo> page = baseMapper.getPage(dto.getPage(), wrapper);
        List<ProductionSchedulingVo> records = page.getRecords();
        if (ObjectUtil.isEmpty(records)) {
            return page;
        }
        //赋值产品信息
        productInfoService.attributeAssign(records, ProductionSchedulingVo::getProductId, (item, productInfo) -> {
            item.setProductName(productInfo.getName());
            item.setProductCode(productInfo.getCustomCode());
            item.setProductLength(productInfo.getLength());
            item.setProductWidth(productInfo.getWidth());
            item.setProductHeight(productInfo.getHeight());
            item.setProductColor(productInfo.getColor());
        });
        return page;
    }

    @Override
    public Map<String, List<ProductionSchedulingVo>> listMap(ProductionSchedulingSelectDto dto) {
        IWrapper<ProductionScheduling> wrapper = getWrapper();
        //权限过滤：生产排程统计
        wrapper.in("ps", ProductionScheduling::getCompanyId, SecurityUtils.getCompanyIds());
        wrapper.eq("ps", ProductionScheduling::getCompanyId, dto.getCompanyId());

        //数据分组
        wrapper.groupBy("ps.scheduling_date", "ps.processes_id");
        //时间范围过滤
        wrapper.ge("ps", ProductionScheduling::getSchedulingDate, dto.getBeginTime());
        wrapper.le("ps", ProductionScheduling::getSchedulingDate, dto.getEndTime());
        //排序
        wrapper.orderByAsc("ps", ProductionScheduling::getSchedulingDate);
        List<ProductionSchedulingVo> list = this.baseMapper.listSumQuantity(wrapper);
        return list.stream().collect(Collectors.groupingBy(ProductionSchedulingVo::getSchedulingDateStr));
    }

    @DSTransactional
    @Override
    public void add(List<ProductionScheduling> dtoList) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        for (ProductionScheduling dto : dtoList) {
            dto.setCompanyId(SecurityUtils.getCompanyId());

            //检查是否重复排程
            long count = this.count(q -> q
                    .eq(ProductionScheduling::getTaskId, dto.getTaskId())
                    .eq(ProductionScheduling::getProcessesId, dto.getProcessesId())
                    .eq(ProductionScheduling::getSchedulingDate, dto.getSchedulingDate())
            );
            if (count > 0) {
                throw new ServiceException(sdf.format(dto.getSchedulingDate()) + "已有排程信息，请前往修改或删除！");
            }
            this.save(dto);

            //获取产品工艺线路
            ProductionOrderDetail productionOrderDetail = produceOrderDetailService.getById(dto.getTaskId());
            Assert.notEmpty(productionOrderDetail, "查询不到生产任务信息！");
            ProductInfo productInfo = productInfoService.getById(productionOrderDetail.getProductId());
            Assert.notEmpty(productInfo, "查询不到产品信息！");
            Technology technology = technologyService.getById(productInfo.getTechnologyId());
            Assert.notEmpty(technology, "查询不到产品工艺信息！");
            String processRoute = technology.getProcessRoute();
            Assert.notEmpty(processRoute, "查询不到产品工艺线路为空！");
            String[] processRouteArr = processRoute.split(",");

            //如果是第一道工序 创建生产领料待出库
            if (Long.parseLong(processRouteArr[0]) == dto.getProcessesId()) {
                createStockWait(dto);
            }
        }
    }

    @Override
    public void edit(ProductionSchedulingDto dto) {
        ProductionScheduling oldPs = this.getById(dto.getId());
        Assert.notEmpty(oldPs, "查询不到排程信息！");

        if (dto.getQuantity() < oldPs.getFinishQuantity()) {
            throw new ServiceException("排程数量不能小于已完工数量！");
        }

//        this.updateById(dto);
    }

    @Override
    public void delete(Long id) {
        ProductionScheduling oldPs = this.getById(id);
        Assert.notEmpty(oldPs, "查询不到排程信息！");

        if (oldPs.getFinishQuantity() > 0) {
            throw new ServiceException("该排程已有完工信息，禁止删除！");
        }

        this.removeById(id);

        //删除生产领料待出库
        StockWait stockWait = stockWaitService.getOne(q -> q.eq(StockWait::getSchedulingId, id));
        if (ObjectUtil.isNotEmpty(stockWait)) {
            long count = stockWaitDetailsService.count(q -> q.eq(StockWaitDetails::getStockWaitId, stockWait.getId()).ne(StockWaitDetails::getReceiptQuantity, 0));
            if (count > 0) {
                throw new ServiceException("该排程生产物料，已领料或部分领料，排程禁止删除!");
            }
            stockWaitService.removeById(stockWait.getId());
            stockWaitDetailsService.remove(q -> q.eq(StockWaitDetails::getStockWaitId, stockWait.getId()));
        }
    }

    @Override
    public List<ProductionOrderDetailVo> schedulingTaskList() {
        IWrapper<Object> wrapper = IWrapper.getWrapper();
        //获取当天排程的任务
        wrapper.eq("DATEDIFF( now(), ps.scheduling_date )", 0);
        wrapper.groupBy("pod.id");
        List<ProductionOrderDetailVo> voList = baseMapper.schedulingTaskList(wrapper);

        //赋值产品信息
        productInfoService.attributeAssign(voList, ProductionOrderDetailVo::getProductId, (item, productInfo) -> {
            item.setProductName(productInfo.getName());
            item.setProductCode(productInfo.getCustomCode());
            item.setProductLength(productInfo.getLength());
            item.setProductWidth(productInfo.getWidth());
            item.setProductHeight(productInfo.getHeight());
            item.setProductColor(productInfo.getColor());
        });

        //赋值任务信息

        return voList;
    }

    @Override
    public List<ProductionSchedulingVo> getProcessesSchedulingList(ProductionSchedulingSelectDto dto) {
        Date schedulingDate = dto.getSchedulingDate();
        if (ObjectUtil.isEmpty(schedulingDate)) {
            schedulingDate = new Date();
        }
        String dayDate = DateUtil.format(schedulingDate, "yyyy-MM-dd");
        return produceOrderDetailMapper.getProcessesSchedulingByDay(dayDate);
    }

    /**
     * 根据排程创建待出库
     */
    @DSTransactional
    private synchronized void createStockWait(ProductionScheduling dto) {
        ProductionOrderDetail task = produceOrderDetailService.getById(dto.getTaskId());
        Assert.notEmpty(task, "查询不到任务信息！");

        //生成待出库
        StockWait stockWait = new StockWait();
        stockWait.setStatus(0);//待出库
        stockWait.setCompanyId(task.getCompanyId());
        stockWait.setBusinessId(dto.getId());
        stockWait.setType(2);//出库

        stockWait.setBusinessType(JournalType.PROD_OUT.getDetailType());//生产任务出库
        stockWait.setBusinessCode(task.getCode());

        stockWait.setContractId(task.getContractId());
        stockWait.setProdOrderId(task.getProduceOrderId());
        stockWait.setProdTaskId(task.getId());
        stockWait.setSchedulingId(dto.getId());
        stockWaitService.saveOrUpdate(stockWait);

        List<StockWaitDetails> stockWaitDetailsList = new ArrayList<>();
        List<ContractProductBomVo> contractProductBomQuantitySum = contractProductBomService.getContractProductBomQuantitySum(IWrapper.getWrapper()
                .eq("cp.id", task.getContractDetailId())
                .ne("cpb.type", 3)//过滤掉工序的数据
        );
        for (ContractProductBomVo contractProductBomVo : contractProductBomQuantitySum) {
            //计算排程需要的量
            ContractProduct cpById = contractProductService.getById(contractProductBomVo.getContractProductId());
            BigDecimal multiply = contractProductBomVo.getQuantity().divide(cpById.getQuantity()).multiply(BigDecimal.valueOf(dto.getQuantity()));

            //创建待出库明细
            Long stockWaitId = stockWait.getId();
            StockWaitDetails stockWaitDetails = new StockWaitDetails();
            stockWaitDetails.setStockWaitId(stockWaitId);
            stockWaitDetails.setReceiptQuantity(BigDecimal.ZERO);
            stockWaitDetails.setContractDetailId(task.getContractDetailId());
            stockWaitDetails.setProdOrderId(task.getProduceOrderId());
            stockWaitDetails.setProdTaskId(task.getId());

            stockWaitDetails.setProductId(contractProductBomVo.getMaterialId());
            stockWaitDetails.setQuantity(multiply);
            stockWaitDetails.setSchedulingId(dto.getId());

            stockWaitDetailsList.add(stockWaitDetails);
        }
        stockWaitDetailsService.saveOrUpdateBatch(stockWaitDetailsList);
    }

}
