package com.fjhx.wms.flow;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.wms.entity.manual.dto.ManualStockDto;
import com.fjhx.wms.entity.manual.po.ManualStock;
import com.fjhx.wms.entity.manual.po.ManualStockDetails;
import com.fjhx.wms.service.manual.ManualStockDetailsService;
import com.fjhx.wms.service.manual.ManualStockService;
import com.fjhx.wms.service.stock.StockService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;

@Component
public class ManualInStockFlow extends FlowDelegate {

    @Autowired
    private ManualStockService manualStockService;
    @Autowired
    private ManualStockDetailsService manualStockDetailsService;
    @Autowired
    private CodingRuleService codingRuleService;
    @Autowired
    private StockService stockService;

    @Override
    public String getFlowKey() {
        return "manual_in_stock_flow";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        ManualStockDto dto = submitData.toJavaObject(ManualStockDto.class);
        dto.setFlowId(flowId);

        if (ObjectUtil.isEmpty(dto.getOpType())) {
            dto.setOpType(1);
        }

        if (dto.getOpType().equals(1)) {
            dto.setCode(codingRuleService.createCode(CodingRuleEnum.MANUAL_IN_STOCK.getKey(), null));
        } else {
            dto.setCode(codingRuleService.createCode(CodingRuleEnum.MANUAL_OUT_STOCK.getKey(), null));
        }

        dto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());

        manualStockService.add(dto);

        return dto.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        manualStockService.update(q -> q
                .eq(ManualStock::getId, businessId)
                .set(ManualStock::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );

        //获取入库数据信息
        ManualStock manualStock = manualStockService.getById(businessId);
        List<ManualStockDetails> manualStockDetailsList = manualStockDetailsService.list(q -> q.eq(ManualStockDetails::getManualStockId, businessId));

        //生成出入库数据
        ManualStockDto stockDto = BeanUtil.copyProperties(manualStock, ManualStockDto.class);
        stockDto.setList(manualStockDetailsList);
        stockService.add(stockDto);
    }

    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        start(flowId, submitData);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        manualStockService.update(q -> q
                .eq(ManualStock::getId, businessId)
                .set(ManualStock::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        manualStockService.cancellation(businessId);
    }
}
